import './JSXColoringProvider.css';

let monaco = null, j = null;

const defaultOptions = {
    isHighlightGlyph: true,
    iShowHover: false,
};

const HIGHLIGHT_SCOPE = {
    ALL: 'ALL',
    IDENTIFIER: 'IDENTIFIER',
    LOCAL: 'LOCAL',
    EXTRA: 'EXTRA',
};

export const JSXTypes = {
    JSXBracket: {
        highlightScope: HIGHLIGHT_SCOPE.LOCAL,
        options: {
            inlineClassName: 'mtk100.Identifier.JsxElement.Bracket',
        },
    },
    JSXOpeningElement: {
        highlightScope: HIGHLIGHT_SCOPE.IDENTIFIER,
        options: {
            inlineClassName: 'mtk101.Identifier.JsxOpeningElement.Identifier',
        },
    },
    JSXClosingElement: {
        highlightScope: HIGHLIGHT_SCOPE.IDENTIFIER,
        options: {
            inlineClassName: 'mtk102.Identifier.JsxClosingElement.Identifier ',
        },
    },
    JSXAttribute: {
        highlightScope: HIGHLIGHT_SCOPE.IDENTIFIER,
        options: {
            inlineClassName: 'mtk103.Identifier.JsxAttribute.Identifier ',
        },
    },
    JSXText: {
        highlightScope: HIGHLIGHT_SCOPE.ALL,
        options: {
            inlineClassName: 'mtk104.JsxElement.JsxText',
        },
    },
    JSXElement: {
        highlightScope: HIGHLIGHT_SCOPE.EXTRA,
        options: (elementName) => (
            {
                glyphMarginClassName: 'mtk105.glyph.Identifier.JsxElement',
                glyphMarginHoverMessage:
                    `JSX Element${elementName ? ': ' + elementName : ''}`
            }
        ),
    },
};

export const configureLocToMonacoRange = (_monaco = monaco, parser = 'babylon') => {
    switch (parser) {
        case 'babylon':
        default:
            return (
                loc,
                startLineOffset = 0,
                startColumnOffset = 0,
                endLineOffset = 0,
                endColumnOffset = 0,
            ) => {
                if (!loc || !loc.start) {
                    return new _monaco.Range(1, 1, 1, 1);
                }
                return new _monaco.Range(
                    startLineOffset + loc.start.line,
                    startColumnOffset + loc.start.column + 1,
                    endLineOffset + loc.end ? loc.end.line : loc.start.line,
                    endColumnOffset + loc.end ? loc.end.column + 1 : loc.start.column + 1,
                );
            };
    }
};

class JSXColoringProvider {
    constructor(
        monacoRef,
        jRef,
        monacoEditor,
        options = defaultOptions,
        editorId
    ) {
        monaco = monacoRef;
        j = jRef;
        this.locToMonacoRange = configureLocToMonacoRange(monaco);
        this.monacoEditor = monacoEditor;
        this.options = options;
        this.editorId = editorId;
    }

    highLightOnDidChangeModelContent = (
        onResolve = ast => ast,
        onError = error => console.error(error),
    ) => {
        this.colorizeCode();
        let highlighterDisposer = this.monacoEditor.onDidChangeModelContent(
            () => this.colorizeCode()
                .then(onResolve)
                .catch(onError)
        );
        return () => {
            if (!highlighterDisposer) {
                return;
            }
            highlighterDisposer.dispose();
            this.monacoEditor.deltaDecorations(
                this.JSXDecoratorIds || [],
                [],
            );
            highlighterDisposer = null;
        };
    };

    colorize = (source, isSourceCode) => {
        return new Promise((resolve) => {
            const ast = isSourceCode ? j(source) : source;
            if (ast) {
                const decorators = this.createJSXElementDecorators(ast);
                for (const jsxType in JSXTypes) {
                    this.createDecoratorsByType(
                        ast,
                        jsxType,
                        JSXTypes[jsxType].options,
                        JSXTypes[jsxType].highlightScope,
                        decorators,
                    );
                }
                this.JSXDecoratorIds =
                    this.monacoEditor.deltaDecorations(
                        this.JSXDecoratorIds || [],
                        decorators,
                    );
                resolve(ast);
            }
        });

    };

    colorizeCode = () => {
        this.colorize(this.monacoEditor.getValue(), true);
    };

    createJSXElementDecorators = (
        ast,
        decorators = [],
        highlighterOptions = this.options,
    ) => {
        ast
            .findJSXElements()
            .forEach(p => {
                const loc = p.value.loc;
                const openingElement = p.value.openingElement;
                let elementName = null;
                if (openingElement) {
                    const oLoc = openingElement.loc;
                    elementName = openingElement.name.name;
                    decorators.push({
                        range: new monaco.Range(
                            oLoc.start.line,
                            oLoc.start.column + 1,
                            oLoc.start.line,
                            oLoc.start.column + 2
                        ),
                        options: JSXTypes.JSXBracket.options,
                    });
                    decorators.push({
                        range: new monaco.Range(
                            oLoc.end.line,
                            oLoc.end.column + (openingElement.selfClosing ? -1 : 0),
                            oLoc.end.line,
                            oLoc.end.column + 1
                        ),
                        options: JSXTypes.JSXBracket.options,
                    });
                }
                const closingElement = p.value.closingElement;
                if (closingElement) {
                    const cLoc = closingElement.loc;
                    decorators.push({
                        range: new monaco.Range(
                            cLoc.start.line,
                            cLoc.start.column + 1,
                            cLoc.start.line,
                            cLoc.start.column + 3
                        ),
                        options: JSXTypes.JSXBracket.options,
                    });
                    decorators.push({
                        range: new monaco.Range(
                            cLoc.end.line,
                            cLoc.end.column,
                            cLoc.end.line,
                            cLoc.end.column + 1
                        ),
                        options: JSXTypes.JSXBracket.options,
                    });
                }

                highlighterOptions.isHighlightGlyph && decorators.push({
                    range: this.locToMonacoRange(loc),
                    options: JSXTypes.JSXElement.options(elementName),
                });
            });
        return decorators;
    };

    createDecoratorsByType = (
        ast,
        jsxType,
        jsxTypeOptions,
        highlightScope,
        decorators = [],
        highlighterOptions = this.options,
    ) => {
        switch (highlightScope) {
            case HIGHLIGHT_SCOPE.IDENTIFIER:
                ast.find(j[jsxType])
                    .find(j.JSXIdentifier)
                    .forEach(p => {
                        const loc = p.value.loc;
                        const options = highlighterOptions.iShowHover ?
                            {...jsxTypeOptions, ...{hoverMessage: `(${jsxType})`}}
                            : jsxTypeOptions;
                        decorators.push({
                            range: this.locToMonacoRange(loc),
                            options
                        });
                    });
                break;

            case HIGHLIGHT_SCOPE.ALL:
            default:
                ast.find(j[jsxType])
                    .forEach(p => {
                        const loc = p.value.loc;
                        const options = highlighterOptions.iShowHover ?
                            {...jsxTypeOptions, ...{hoverMessage: `(${jsxType})`}}
                            : jsxTypeOptions;
                        decorators.push({
                            range: this.locToMonacoRange(loc),
                            options
                        });
                    });
        }

        return decorators;
    };
}

export default JSXColoringProvider;